import glob
import logging
import os
import pathlib
import shutil
import sys
import tarfile
from typing import Dict

from .interfaces import ConveyorIF

"""
Deliver weblog and standard capability (i.e. standard calibration or imaging) results to QA area for analyst access
"""

vlapipe_id = 6000


class DeliveryConveyor(ConveyorIF):
    def __init__(self, settings: Dict[str, str]):
        self.logger = logging.getLogger("conveyor")
        self.settings = settings

    def delivery(self):
        self.cache_weblog()
        self.move_subdirectories_to_qa2()

    def cache_weblog(self):
        self.logger.info("Caching Weblog!")
        weblog_location = (
            self.settings["weblog_cache_area"] + "/" + self.settings["destination_subdir"]
        )

        # make new cache directory with name matching workspaces parent directory and unzip weblog into it
        os.makedirs(weblog_location)
        if pathlib.Path("./products/weblog.tgz").exists():
            # shutil.copy("./products/weblog.tgz", weblog_location)
            tar = tarfile.open("./products/weblog.tgz")
            # extract to cache area
            tar.extractall(path=weblog_location)
            # extract in place
            tar.extractall(path="./products")
            tar.close()
        elif self._get_pipeline_dir():
            self.logger.info("WARNING! No weblog.tgz, copying pipeline directory instead.")
            shutil.copytree("./products/pipeline-*", weblog_location)
        else:
            self.logger.info("ERROR: Neither weblog directory or tar file found! Exiting.")
            sys.exit(1)

    @staticmethod
    def _get_pipeline_dir() -> bool:
        if not glob.glob("./products/pipeline-*"):
            return False
        return True

    def move_subdirectories_to_qa2(self):
        self.logger.info("Moving subdirectories to QA area!")
        qa_path = self.settings["qa_delivery_area"] + "/" + self.settings["destination_subdir"]
        if not pathlib.Path(qa_path).exists():
            os.makedirs(qa_path)
            os.chown(qa_path, vlapipe_id, vlapipe_id)
        else:
            self.logger.info(f"Delivering to existing {qa_path}:")
            self.logger.info(os.listdir(qa_path))

        contents = glob.glob("*/", recursive=True)

        # deliver all directories before creating symlinks - ensures delivery if links fail
        for directory in contents:
            src = "./" + directory
            dest = qa_path + "/" + directory
            shutil.copytree(src, dest)

        # link results back to intended destination
        for directory in contents:
            src = qa_path + "/" + directory
            os.system("chown -R 6000:6000 " + src)
            self.symlink_qa(directory, src)

    def symlink_qa(self, dir_name: str, src: pathlib.Path):
        ws_lustre_spool = self.settings["destination_dir"] + "/" + dir_name
        self.logger.info(f"Linking QA2 and workspaces parent directory for directory {dir_name}!")
        pathlib.Path(ws_lustre_spool).symlink_to(src, target_is_directory=True)
        self.logger.info("Symlink Created!")

    def convey(self):
        self.logger.info("RUNNING STANDARD CALIBRATION DELIVERY! ")
        self.delivery()
